@@ -94,6 +94,7 @@ NULL
 #'
 #' @import Matrix
 #' @importFrom future.apply future_lapply
+#' @importFrom future nbrOfWorkers
 #' @importFrom MASS theta.ml theta.mm glm.nb negative.binomial
 #' @importFrom stats glm glm.fit df.residual ksmooth model.matrix as.formula approx density poisson var bw.SJ
 #' @importFrom utils txtProgressBar setTxtProgressBar capture.output
@@ -141,14 +142,19 @@ vst <- function(umi,
                 show_progress = NULL) {
   if (!is.null(vst.flavor)){
     if (vst.flavor == "v2"){
-      if (verbosity>0){
-        message("vst.flavor='v2' set, setting model to use fixed slope and exclude poisson genes.")
+      if (verbosity > 0){
+        message("vst.flavor='v2' set. Using model with fixed slope and excluding poisson genes.")
       }
       glmGamPoi_check <- requireNamespace("glmGamPoi", quietly = TRUE)
       method <- "glmGamPoi_offset"
       if (!glmGamPoi_check){
-        message('`vst.flavor` is set to "v2" but could not find glmGamPoi installed.
-                Please install the glmGamPoi package. See https://github.com/const-ae/glmGamPoi for details.')
+        message("`vst.flavor` is set to 'v2' but could not find glmGamPoi installed.\n",
+                "Please install the glmGamPoi package for much faster estimation.\n",
+                "--------------------------------------------\n",
+                "install.packages('BiocManager')\n",
+                "BiocManager::install('glmGamPoi')\n",
+                "--------------------------------------------\n",
+                "Falling back to native (slower) implementation.\n")
         method <- "nb_offset"
       }
       exclude_poisson <- TRUE
@@ -257,7 +263,7 @@ vst <- function(umi,
     overdispersion_factor <- genes_var - genes_amean
     overdispersion_factor_step1 <- overdispersion_factor[genes_step1]
     is_overdispersed <- (overdispersion_factor_step1 > 0)
-    if (verbosity > 0) {
+    if (verbosity > 1) {
       message(paste("Total Step 1 genes:",
                     length(genes_step1)))
       message(paste("Total overdispersed genes:", sum(is_overdispersed)))
@@ -363,7 +369,7 @@ vst <- function(umi,
     if (min_variance == "umi_median"){
       # Maximum pearson residual for non-zero median UMI is 5
       min_var <- (get_nz_median2(umi) / 5)^2
-      if (verbosity > 0) {
+      if (verbosity > 1) {
         message(paste("Setting min_variance based on median UMI: ", min_var))
       }
       arguments$set_min_var <- min_var
@@ -597,7 +603,12 @@ get_model_pars <- function(genes_step1, bin_size, umi, model_str, cells_step1,
     index_lst <- split(index_vec, ceiling(index_vec/genes_per_worker))
 
     # the index list will have at most n_workers entries, each one defining which genes to work on
-    par_lst <- future_lapply(
+    if (nbrOfWorkers() == 1){
+      my.lapply <- function(X, FUN, future.seed = TRUE) lapply(X, FUN)
+    } else {
+      my.lapply <- future_lapply
+    }
+    par_lst <- my.lapply(
       X = index_lst,
       FUN = function(indices) {
         umi_bin_worker <- umi_bin[indices, , drop = FALSE]
@@ -674,7 +685,7 @@ get_model_pars <- function(genes_step1, bin_size, umi, model_str, cells_step1,
 
     # if the naive and estimated MLE are 1000x apart, set theta estimate to Inf
     diff_theta_index <- rownames(model_pars[model_pars[genes_step1, "diff_theta"]< 1e-3,])
-    if (verbosity>0){
+    if (verbosity > 1){
       message(paste("Setting estimate of ", length(diff_theta_index), "genes to inf as theta_mm/theta_mle < 1e-3"))
     }
     # Replace theta by infinity
@@ -687,6 +698,7 @@ get_model_pars <- function(genes_step1, bin_size, umi, model_str, cells_step1,
 }
 
 get_model_pars_nonreg <- function(genes, bin_size, model_pars_fit, regressor_data, umi, model_str_nonreg, cell_attr, verbosity) {
+  
   bin_ind <- ceiling(x = 1:length(x = genes) / bin_size)
   max_bin <- max(bin_ind)
   if (verbosity > 1) {
@@ -697,9 +709,14 @@ get_model_pars_nonreg <- function(genes, bin_size, model_pars_fit, regressor_dat
     genes_bin <- genes[bin_ind == i]
     mu <- tcrossprod(model_pars_fit[genes_bin, -1, drop=FALSE], regressor_data)
     umi_bin <- as.matrix(umi[genes_bin, ])
+    if (nbrOfWorkers() == 1){
+      my.lapply <- function(X, FUN, future.seed = TRUE) lapply(X, FUN)
+    } else {
+      my.lapply <- future_lapply
+    }
     model_pars_nonreg[[i]] <- do.call(
       rbind,
-      future_lapply(X = genes_bin,
+      my.lapply(X = genes_bin,
                     FUN = function(gene) {
                       fam <- negative.binomial(theta = model_pars_fit[gene, 'theta'], link = 'log')
                       y <- umi_bin[gene, ]
@@ -752,7 +769,7 @@ reg_model_pars <- function(model_pars, genes_log_gmean_step1, genes_log_gmean, c
 
     poisson_genes_step1 <- genes_step1[overdispersion_factor_step1<=0]
 
-    if (verbosity>0){
+    if (verbosity > 1){
       message(paste("# of step1 poisson genes (variance < mean):",
                     length(poisson_genes_step1)))
       message(paste("# of low mean genes (mean < 0.001):", length(low_mean_genes)))
@@ -765,7 +782,7 @@ reg_model_pars <- function(model_pars, genes_log_gmean_step1, genes_log_gmean, c
     overdispersed_genes_step1 <- setdiff(genes_step1, poisson_genes_step1)
 
 
-    if (verbosity>0){
+    if (verbosity > 1){
       message(paste("Total # of Step1 poisson genes (theta=Inf; variance < mean):",
                     length(poisson_genes_step1)))
       message(paste("Total # of poisson genes (theta=Inf; variance < mean):",
@@ -844,7 +861,7 @@ reg_model_pars <- function(model_pars, genes_log_gmean_step1, genes_log_gmean, c
 
 
   if (exclude_poisson){
-    if (verbosity > 0) {
+    if (verbosity > 1) {
       message('Ignoring theta inf genes')
     }
     overdispersed_genes <- setdiff(rownames(model_pars), all_poisson_genes)
@@ -930,10 +947,10 @@ reg_model_pars <- function(model_pars, genes_log_gmean_step1, genes_log_gmean, c
   all_genes <- rownames(model_pars_fit)
   # browser()
   if (exclude_poisson){
-    if (verbosity > 0) {
+    if (verbosity > 1) {
       message(paste('Replacing fit params for', length(all_poisson_genes),  'poisson genes by theta=Inf'))
     }
-    for (col in colnames(model_pars_fit)){
+    for (col in intersect(colnames(x = model_pars_fit), colnames(x = vst_out_offset)) ){
       stopifnot(col %in% colnames(vst_out_offset))
       model_pars_fit[all_poisson_genes, col] <- vst_out_offset[all_poisson_genes, col]
     }
@@ -942,7 +959,7 @@ reg_model_pars <- function(model_pars, genes_log_gmean_step1, genes_log_gmean, c
   if (fix_intercept){
     # Replace the fitted intercepts by those calculated from offset model
     col <- "(Intercept)"
-    if (verbosity > 0) {
+    if (verbosity > 1) {
       message(paste0('Replacing regularized parameter ', col, ' by offset'))
     }
     gene_mean <- rowMeans(umi)
